package mactrans.tools;

import java.util.ArrayList;

/**
 * This tokenizer distinguishes between interpunctive and
 * alphanumerical tokens. It could also be extended to
 * support statistical purposes.
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.4 $
 * 
 * <pre>
 * $Log: MacTransTokenizer.java,v $
 * Revision 1.4  2005/02/01 15:36:23  dfriehe
 * fixed various bugs (index counting was wrong)
 *
 * Revision 1.3  2005/01/26 13:48:38  jkieneck
 * ...and done it..  ;-)
 *
 * Revision 1.2  2005/01/26 13:45:42  jkieneck
 * added methods (todo) for iterator-like support
 *
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */

public class MacTransTokenizer {
	/* local vars */
	private String input;
	private ArrayList tokens;
	private int cPos;
	
	/**
	 * Creates a tokenized version of a given String input
	 * 
	 * @param input a String to put into the tokenizer
	 */
	public MacTransTokenizer( String input ) {
		this.input = input;
		if ( input == null ) {
			throw( new NullPointerException( "MacTransTokenizer: input is null!" ) );
		}
		tokens = new ArrayList();
		tokenize();
		cPos = 0;
	}
	
	/**
	 * Tells whether the tokenizer contains tokens or not
	 * 
	 * @return true, if there is at least one token in it
	 */
	public boolean isEmpty() {
		return ( getTokens() == 0 );
	}
	
	/**
	 * Returns the total amount of tokens contained in the
	 * tokenizer, disregarding the symbolic type
	 * 
	 * @return the amount of total tokens
	 */
	public int getTokens() {
		return tokens.size();
	}
	
	/**
	 * Returns the total amount of interpunctions contained
	 * in the tokenized string.
	 * 
	 * @return the total amount of interpunctions in the tokenized string
	 */
	public int getInterpunctions() {
		int result = 0;
		for ( int i = 0; i < tokens.size(); i ++ ) {
			if ( isInterpunction( (String) tokens.get( i ) ) ) {
				result++; 
			}
		}
		return result;
	}
	
	/**
	 * Returns the total amount of alphanumerical tokens contained
	 * in the tokenized string.
	 * 
	 * @return the total amount of alphanumerical tokens in the tokenized string
	 */
	public int getWords() {
		int result = 0;
		for ( int i = 0; i < tokens.size(); i ++ ) {
			if ( isAlphaNumerical( (String) tokens.get( i ) ) ) {
				result++; 
			}
		}
		return result;
	}
	
	/**
	 * Returns the token at the position specified by index, disregarding
	 * of which type the token is. Please note, that the first token
	 * in this tokenizer is indexed via the integer 1, as a few people wished.
	 * 
	 * @param index the position of the token, >=1 and <= getTokens()
	 * @return the token addressed via index
	 */
	public String getToken( int index ) {
		return ((String) tokens.get( index ));
	}
	
	/**
	 * Returns the interpunction token specified by index. Please note,
	 * that the first interpunction in this tokenizer is indexed via the
	 * integer number 1.
	 * 
	 * @param index the interpunction token, >=1 and <= getInterpunctions()
	 * @return the interpunction addressed via index
	 */
	public String getInterpunction( int index ) {
		String result = new String();
		int cindex = 0;
		int i = 0;
		while ( i < getTokens() && cindex < index ) {
			if ( isInterpunction( (String) tokens.get( i ) ) ) {
				cindex++;
				if ( cindex == index ) {
					result = (String) tokens.get( i );
				}
			}
			i++;
		}
		return result;
	}
	
	/**
	 * Returns the alphanumerical token specified by index. Please note,
	 * that the first alphanumerical token/word in this tokenizer is
	 * indexed using the integer number 1, because a few people wanted
	 * it to work like this.
	 * 
	 * @param index the alphanumerical token/word, >=1 and <= getWords()
	 * @return the alphanumerical token/word addressed via index
	 */
	public String getWord( int index ) {
		String result = new String();
		int cindex = -1;
		int i = 0;
		while ( i < getTokens() && cindex < index ) {
			if ( isAlphaNumerical( (String) tokens.get( i ) ) ) {
				cindex++;
				if ( cindex == index ) {
					result = (String) tokens.get( i );
				}
			}
			i++;
		}
		return result;
	}
	
	public boolean hasNext() {
		return ( cPos < getWords() );
	}
	
	public String getNext() {
		String result = null;
		if ( hasNext() ) {
			result = getWord( cPos );
			cPos++;
		}
		return result;
	}
	
	/**
	 * Very simple method to detect whitespace chars
	 * 
	 * @param t a string (or better char'd string) to investigate
	 * @return true, if t is " "
	 */
	public boolean isWhitespace( String t ) {
		return t.equals( " " ) || t.equals( "\n" ) || t.equals( "\t" ) || t.equals( "\r" );
	}
	
	/**
	 * Simple method to detect numerical chars
	 * 
	 * @param t a string to investigate
	 * @return true, if t begins with ["0".."9"]
	 */
	public boolean isNumerical( String t ) {
		char c = t.charAt( 0 );
		return ( c >= '0' && c <= '9' );
	}
	
	/**
	 * Simple method to detect interpunction chars
	 * 
	 * @param t a string to investigate
	 * @return true, if t begins with one of [.,:;!?]
	 */
	public boolean isInterpunction( String t ) {
		boolean result = false;
		if ( t.length() > 0 ) { 
			char c = t.charAt( 0 );
			result = ( c == '.' || c == ',' || c == ':' || c == ';' || c == '!' || c == '?' );
		}
		return result;
	}
	
	/**
	 * Simple method to detect alpha chars
	 * 
	 * @param t a string to investigate
	 * @return true, if t begins with [A-Za-z]
	 */
	public boolean isAlpha( String t ) {
		return ( !isInterpunction( t ) && !isNumerical( t ) && !isWhitespace( t ) );
	}
	
	/**
	 * Simple method to detect alphanumerical chars
	 * 
	 * @param t a string to investigate
	 * @return true, if isAlpha or isNumerical ;-)
	 */
	public boolean isAlphaNumerical( String t ) {
		return ( isAlpha( t ) || isNumerical( t ) );
	}
	
	/**
	 * This method sets up the tokens by reading through the
	 * "input" string.
	 *
	 */
	public void tokenize() {
		String temp = new String();
		String tChar = new String();
		int len = input.length();
		int i = 0;
		
		while ( i < len ) {
			// scan for the beginning of a token
			while ( i < len && isWhitespace( input.substring( i, i + 1 ) ) ) {
				i++;
			}
			
			// assume we are not at the end of the string
			if ( i < len ) {
				// meaning: this is the beginning of a new token
				// we want to make a difference between real words
				// interpunction symbols
				if ( isInterpunction( input.substring( i, i + 1) ) ) {
					// let's go for interpunction
					temp = "";
					while ( i < len && isInterpunction( input.substring( i, i + 1 ) ) ) {
						temp += input.substring( i, i + 1 );
						i++;
					}
					tokens.add( temp );
				} else {
					// let's go for alphanumerical data
					temp = "";
					while ( i < len && isAlphaNumerical( input.substring( i, i + 1 ) ) ) {
						temp += input.substring( i, i + 1 );
						i++;
					}
					tokens.add( temp );
				}
			}
		}
	}
}
